package fr.username404.snowygui.gui.feature

import com.mojang.blaze3d.vertex.PoseStack
import fr.username404.snowygui.config.Configuration
import fr.username404.snowygui.gui.ColoredElement
import fr.username404.snowygui.gui.Renderable
import fr.username404.snowygui.utils.FontUtil
import fr.username404.snowygui.utils.RenderingUtil
import net.minecraft.client.Minecraft

@ButtonInfo(Category.HUD)
object Keystrokes: ButtonImplWithHud() {
    var currentPosition: Double by Configuration; internal set
    @Suppress("UNUSED")
    enum class Position(internal val value: Double) {
        LEFT(180.0),
        RIGHT(-120.0)
    }
    private val hud = object : ColoredElement(x = 135.0, y = 20.0, height = 36, width = 36, color = Colors.BLACK(), opacity = 0.5F) {
        override var x: Double get() = (Minecraft.getInstance().window.guiScaledWidth / 2.0) - currentPosition; set(value) = Unit
        override var y: Double
            get() = Minecraft.getInstance().window.guiScaledHeight.toDouble() - yOrigin
            set(value) = Unit
        private val keysArray by lazy {
            Minecraft.getInstance().options.run {
                arrayOf(keyLeft, keyDown, keyRight, keyUp) to arrayOf(0.5, 0.5, 0.5, 0.5)
            }
        }
        private val minimumOpacity = opacity.toDouble()
        private fun getDynamicOpacity(i: Int): Double {
            val dynamicOpacity = keysArray.second
            if (keysArray.first[i].isDown) {
                if (dynamicOpacity[i] < 1) {
                    dynamicOpacity[i] += 0.05
                }
            } else {
                if (dynamicOpacity[i] > minimumOpacity) {
                    dynamicOpacity[i] -= 0.05
                }
            }
            return dynamicOpacity[i]
        }

        private fun drawKey(x: Double, y: Double, key: Int, poseStack: PoseStack? = null) {
            RenderingUtil.prepareDraw()
            RenderingUtil.drawRectangle(
                x, y, height / 2, height / 2,
                color = color, opacity = getDynamicOpacity(key).toFloat()
            )
            RenderingUtil.endDraw()
            poseStack?.let {
                FontUtil.drawScaled(
                    it, keysArray.first[key].translatedKeyMessage.string.uppercase(),
                    x + height.toDouble() / 5.5, y + height.toDouble() / 6.5,
                    scaleFactor = 1F, color = Colors.WHITE
                )
            }
        }
        private fun getNewPos(i: Int) = x + (i * (2 + (height / 2)))
        override fun render(poseStack: PoseStack?) = with(RenderingUtil) {
            for (i in 0 until 3) {
                drawKey(getNewPos(i), y, i, poseStack)
            }
            drawKey(getNewPos(1),  y - (height / 2) - 2, 3, poseStack)
        }
    }
    override val hudRenderLambda = Renderable {
        hud.display(it)
    }
    init {
        if (!Position.values().map { it.value }.contains(currentPosition)) {
            currentPosition = Position.LEFT.value
        }
    }
}